﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinSchedule.TimelineView;
using Infragistics.Win.UltraWinScrollBar;

namespace WTLV_NumberOfDaysVisible
{
    class CustomCreationFilter : IUIElementCreationFilter
    {
        int headerWidth = -1;
        int scrollBarWidth = -1;

        double offset;

        public double Offset
        {
            get { return offset; }
            set { offset = value; }
        }

        public int HeaderWidth
        {
            get { return headerWidth; }
            set { headerWidth = value; }
        }

        public int ScrollBarWidth
        {
            get { return scrollBarWidth; }
            set { scrollBarWidth = value; }
        }

        public void AfterCreateChildElements(UIElement parent)
        {
            if (parent is ScrollBarUIElement)
            {
                if (scrollBarWidth != -1)
                    return;

                var context = (Orientation)parent.GetContext(typeof(Orientation));

                if (context == Orientation.Vertical)
                {
                    scrollBarWidth = parent.Rect.Width;
                }
            }

            if (parent is OwnerHeaderUIElement)
            {
                if (headerWidth == -1)
                {
                    headerWidth = parent.Rect.Width;
                }
            }

            if ( offset > 1 && parent is ActivityAreaUIElement)
            {
                AdjustLastElement(parent);
            }

            if (offset > 1 && parent is ColumnHeaderAreaUIElement)
            {
                AdjustLastElement(parent);
            }
        }

        private void AdjustLastElement(UIElement parent)
        {
            var parentRect = parent.Rect;
            var parentElements = parent.ChildElements.ToList();

            var elements = parentElements.Where(el => el.Rect.Right < parentRect.Right
                && el.Rect.Right + offset > parentRect.Right).ToList();

            if (elements != null && elements.Count > 0)
            {
                var hiddenElements = parentElements.Where(el => el.Rect.Right >= parentRect.Right).ToList();

                foreach (var element in elements)
                {
                    element.Rect = new Rectangle(element.Rect.Location, new Size(element.Rect.Width +
                             (parentRect.Right - element.Rect.Right), element.Rect.Height));
                }

                if (hiddenElements != null && hiddenElements.Count > 0)
                {
                    foreach (var hiddenElement in hiddenElements)
                    {
                        hiddenElement.Rect = new Rectangle(parentRect.Right, parentRect.Top, 0, 0);
                    }
                }

            }
        }

        public bool BeforeCreateChildElements(UIElement parent)
        {
            return false;
        }
    }
}
